# GOOGLE AUTH

A self-contained way to get access tokens for interrogating Google APIs.

In particular, its intended to be used as an Installed App that doesn't need a non-Google server.

## Usability Goals

* Let users log into multiple Google accounts in one Chrome session. (Extension-using power users in particular have both personal and professional Gmail accounts, but find it a hindrance to constantly switch Chrome Profiles.)
* Do not be repeatedly asked to authorise the extension.

These goals led to choosing the chrome.identity.launchWebAuthFlow rather than chrome.identity.getAuthToken (which can only be used with the primary Chrome Google account), and to use 'offline' OAuth with refresh tokens to persist the authorisation.

## Security Goals

* Never let OAuth tokens granted to the extension reach the extension's server, even during the authorisation flow. (Even if the server has passed the necessary Security Assessment, it remains a large theoretical attack vector as rogue employees can readily circumvent any security hardening to access server data.)
* Prevent MITM attacks that steal tokens during the authorisation flow.
* Never leak the refresh token outside of the runtime closure.
* Prevent the extension, or its OAuth client credentials, being used to enable phishing attacks.


## Overall Flow

It's modelled on https://developers.google.com/identity/protocols/OAuth2InstalledApp.


**Setting up the OAuth Client Credentials**

The only significant difference to the OAuth2InstallApp docs is that the _Application Type_ of the OAuth Client ID needs to be _Web Application_ (as opposed to Android/iOS/Chrome App/Other), because it's currently the only way to lockdown the _redirect_uri_, which is pivotal to making the chrome.identity.launchWebAuthFlow secure.

The trade off is it exposes the _client_secret_ publically, and while the OAuth2InstalledApp docs linked above accept that trade off if PKCE is used, the Application Type doesn't let Google the provider know the client_secret is public, which might result in mis-calibrated tolerances for suspicious activity. (See Future Flows below).

**The Authorization flow**

1. This Authorization Flow can only be invoked by code running in response to a DOM event generated by a user interaction with a listener set up in the extension's Content Script context, to make phishing attacks that attempt to invoke the extension's authorisation flow much harder. TODO
2. The Authorization Credentials are obtained by securely passing the user to Google-owned pages to enter their credentials using chrome.identity.launchWebAuthFlow, and the Authorization code is passed back to the extension with a redirect_uri locked down during the creation of the OAuth Client Credential to https://<extensionid>.chromiumapp.org/oauth2, that ensures only the extension with the provided ID can ever receive the returned Authorization Code. The use of a state test prevents CSRF.
3. Using a PKCE challenge established in step 1, a POST request is sent to exchange the Authorization Code for a Refresh Token.

The use of the locked down redirect_uri that can only be accessed by this extension's unspoofable GUID means that even if the OAuth Credentials are obtained by an attacker they can't receive the Authorization Code (or an access token if they tried an Implicit Grant Flow); and the use of PKCE means that if they intercept and try to use the Authorization Code with a MITM they still can't exchange it for a Refresh Token.

**How the Refresh Token is secured**

* The googleAuth runtime code is wrapped in a closure, kept in the Background Page of the Extension, with no methods that expose the refresh token.
* The refresh token is stored on the hard drive in a way that's only accessible by this extension using Chrome's native storage mechanism. As an extra step against generic malware that sweeps hard drives for data that looks like tokens, the Refresh Token is broken into a key and an encrypted output using a One Time Pad / Vernom Cipher, and stored apart from each other in chrome.storage and the Background Page's document.cookie which is additionally encrypted on the hard drive by Chrome (but targetted malware in the user's environment could still decrypt it - see Future Flows for how this might be mitigated with a usability trade off).
* To help Chrome Extension Reviewers verify that the refresh token isn't leaked to 3rd party servers, every call to a 3rd party server, or to an encryption mechanism, must clearly ask the googleAuth module to check no fingerprint of the refresh token is in the data being uploaded/encrypted. (See #FINGERPRINT_REFRESH_TOKENS in code). 

**How the extension accesses Google APIs**

1. It asks the googleAuth module for a short-lived access token, to authenticate the API request.
2. If the access token has expired, the module asks for a new access token using the Refresh Token, and returns the new access token.
3. If it isn't valid for the Scope required, the user is shown an explanation modal, followed by the above Authorization Flow. Then it asks again for the short-lived access token.

**Mitigating the exposure of the client_secret and responding to suspicious behaviour**
* The extension must use Cross-Account Protection (https://developers.google.com/identity/risc). The extension will act to shut down the client_id if a mass breach is detected. Or if a single user is compromised, the extension must be notified to automtically revoke the token, or if they no longer use the extension there will be a process to email them with instructions to manually revoke the token. TODO
* It is understood that the client_id must not be used for any other authorization flow, as the use of the compromised client_secret gives a false sense of security to both the developer of the flow, and to Google the provider (who would have signals that the new and legitimate use of client_secret without PKCE is acceptable, rather than being anomalous suspicious behaviour). Suffix the OAuth Client Credential name with SINGLE_EXTENSION_USE_ONLY.

## How the extension's server authenticates users to access its non-Google data

TODO (after generating the API login end point)

1. The user submits their Google credentials to sign into the extension using the Authorization Flow above.
2. The extension generates an ID Token using the Refresh Token, and passes this to the extension's server.
3. The extension's server verifies the short-lived ID Token, and generates its own session token that the extension uses for all subsequent interactions.

Access or Refresh Tokens are not sent to the server to achieve this.

Never use access/refresh/id tokens in URLs, as it may cause leakage in browser history or referrer headers.

## Making use of a server to provide features that need OAuth tokens to access restricted Google scopes

* **Warning!** It is not permitted to enable OAuth on the extension's server until the server and organization have passed a Google sanctioned Security Assessment.
* The decision to let OAuth Tokens pass through the server must still not be taken lightly, as the attack vector remains whereby rogue employees could relatively easily steal the Refresh Tokens. If the server doesn't need offline access, keep the Refresh Token in the client and grant the server the short-lived access token. Or consider only providing an ID token if it just needs to authenticate the user.
* Once the extension's server can be used to grant the extension OAuth tokens, the extension will not use its webAuthFlowDirectOffline authMethod, as if there is a server flow available, there's no need for it to compromise the client_secret. (A new authMethod could be created that asks the server to complete the exchange of the Authorization Code for a Refresh Token).
* Never use the same client_id that was used in the extension's webAuthFlowDirectOffline authMethod, as the client_secret has been exposed.


## Alternative Flows, and their trade offs
* Use an OAuth Client Credential with the _Application Type_ of _Chrome App_, combined with the getAuthTokenDirectOnline authMethod. This lets Chrome handle the bulk of the security (only short-lived access tokens can be compromised), but with the major usability costs of forcing the user to sign into and frequently switch to a new Chrome Profile for each Google account they have, and having to regularly re-authorize consent for the extension.
* Complete the exchange of the Authorization Code for a Refresh Token via the extension's own server, to keep the client_secret secret and to enable Google to lock down the OAuth Client Credentials to an Authorized Javascript Origin. However, it is not permitted without a Security Assessment being completed on the server, and still opens the extension to the possibility of a relatively easy attack by rogue employees to steal tokens from the server.

## Future Flows, when the technology allows or usability conventions permit
* If Google introduces a new OAuth Client Credential _Application Type_, that enables a locked down redirect_uri, but acknowledges the client_secret is exposed and recalibrates their tolerances to suspicious behaviour signals accordingly, use that immediately.
* To protect the Refresh Token from theft by targetted malware while at rest on the hard drive, add a preliminary step to the Authorization Flow to ask the user to enter a password for this extension, that is used to generate the key for the One Time Pad / Vernom Cipher, so that no data is committed to the hard drive that can be used to reveal the Refresh Token. The major usability trade off here is that the user would have to re-enter their extension password for every new session of Chrome. Given the unlikely nature of targetted malware against this extension (if malware has breached the user there are more attractive targets to go for), this usability trade off isn't worth it at present.

## Disregarded Flows
* The Implicit Grant Flow should not be used (aka the webAuthFlowDirectOnline authMethod). This is the official line of the OAuth Working Group. It was conceived pre-CORS, is insecure due to not using second step of the Authorization Code + PKCE, and for returning the sensitive access_token to the redirect_uri as a hash which may cause leakage via browser history.
* Any attempt to keep the client_secret secret by storing outside the app (e.g. on a server) and providing it only at runtime is trivial for an attacker to steal, as they can add breakpoints to the Background Page to extract it.



----



## Attack Vectors - Mitigated

**Extension providers server is breached for OAuth tokens**
The server is a single point of failure for all users, and attackers might include rogue employees with ready access.
* Therefore try to avoid any solution that involves sending tokens through the extension's server.

**Cross-Site Request Forgery, listenable redirect_uris and other MITM attacks**
For installed apps to do OAuth, the provider that accepts the user entering their credentials (which cannot be installed app) must communicate back with the installed app to complete the process. Even though its assumed Chrome's chrome.identity.launchWebAuthFlow and chrome.identity.getRedirectURL are secure, this is so high risk that the extra step of validating the communication hasn't been intercepted.
* Use PKCE and state in requests for Authorization Credentials and to exchange the returned Authorization Code for a Refresh Token.

**Refresh Token stolen from the Extension**
The refresh token is stored on the user's device. An attacker, most likely via malware, could potentially harvest the refresh tokens by attacking the extension or hard drive (although malware has so many ways to compromise a user's data that this is unlikely to be the most attractive). An attacker could also use the Developer Tools to add break points into the extensions functions and extract data in transit to steal sensitive data (e.g. client_secret).
* The refresh token is not stored intact on the hard disk. It's split into a key and an encrypted version (Vernom Cipher, aka One Time Pad), and stored in two places, and recombined at run time. This only helps make it a little harder for malware to harvest tokens (the malware would have to know precisely how the extension works), and prevent against the unlikely event that a Chrome bug leaked data from one storage place (e.g. to another extension), but doesn't solve the attack vector. A more secure solution, but with a usability trade off, would be to increase the user authorisation flow to ask for a password that they remember, which forms the key for the Vernom Cipher, and thus the refresh token is never actually stored on the hard drive.
* The googleAuth code must be run in the Extension Background Page, and the refresh_token kept within the module/closure, so no other code running in the user's environment can access it.

**The Chrome Extension ID is spoofed**
Renders the redirect_uri loopback to https://<extensionid>.chromiumapp.org/oauth2 insecure.
* The Chrome Web Store forces all extensions to use it for distribution, and has taken responsibility using public/private keys to ensure the ID cannot be spoofed by other addons, even in development mode (which would only be possible with hash collision).

**The extension uploads the refresh_token to a 3rd party server**
* Google reviews all updates. So the reviewer can verify that no code will leave, the Extension should make it explicitly clear what data is sent to a server, by reducing the surface area where this happens (contained in small modules that don't depend on other things, any modules that call them must also be self contained and easy to understand, and using the fewest fetch or XMLHttpRequest objects possible), type checking the data, provide output to the console.log for easy auditing, having uploads occur after user activity (TODO IS THIS POSSIBLE WITH USER ACTIVITY? CREATE A SELF CONTAINED CLOSURE AT THIS POINT?), and a strong Content Security Policy (TODO).
* Possibly provide a function in googleAuth to test any data for the presence of a refresh token.

**Phishing attacks**
In particular, using the trust users have in the extension's brand to go through the flow, if presented by an attacker using stolen OAuth client credentials, or otherwise inducing the extension to start the flow but stealing the returned tokens.
* The locked-down redirect_uri back to https://<extensionid>.chromiumapp.org/oauth2 prevents an attacker using the client_id to begin the process.
* The authorisation code runs in the Background Page of the Chrome extension, within a closure, which Chrome protects from outside invocation. The authorisation flow can only be invoked from the Content Script in response to a user driven DOM event (e.g. clicking a button). TODO.

## Attack Vectors - Tolerated

**client_id and client_secret is exposed**
It shorterns the attack by one step: if the attacker can phish the user into authenticating to get an Authorization Credentials, they can use the client_secret to exchange it for a refresh token.
However, as an installed app we must assume the client_secret is public. Currently, there's no way (other than being an OAuth type of Chrome App, which has usability issues), to tell Google as the provider to assume that client_secret is public. Watch for changes in policy on this, as if we notify them, they might be able to reduce their security tolerance for unexpected behaviour.
* Lock down the OAuth Type to use a redirect_uri that calls https:<extension_id>.chromiumapp.org/oauth2, as created in chrome.identity.getRedirectURL to make it impossible for the attacker to create the Authorization Credentials.
* Use state and PKCE, as discussed at https://developer.okta.com/blog/2019/05/01/is-the-oauth-implicit-flow-dead . TODO -> WHY DOES THIS HELP HERE?
* If it's possible to tolerate the usability issues of only needing to authenticate the signed-in-Chrome user, use getAuthTokenDirectOnline, as Google knows to lock it down only to this extension ID, as part of the Client ID creation.
* Use Cross-Account Protection https://developers.google.com/identity/risc to react rapidly to possible breaches.
* If it becomes possible in the future to have an OAuth Type that lets us lock down a redirect_uri, but also tell Google the provider that the client_secret is public - which would enable them to reduce the security tolerances for suspicious activity on this Client ID, then switch to that.


## Further Reading
* https://developers.google.com/identity/protocols/OAuth2InstalledApp
* https://developers.google.com/identity/protocols/OpenIDConnect#createxsrftoken
* https://auth0.com/docs/api-auth/tutorials/authorization-code-grant-pkce
* https://developer.okta.com/blog/2019/05/01/is-the-oauth-implicit-flow-dead
* https://security.stackexchange.com/questions/182873/why-isnt-pkce-encouraged-for-single-page-apps
* https://security.stackexchange.com/questions/175465/what-is-pkce-actually-protecting
* https://medium.com/securing/what-is-going-on-with-oauth-2-0-and-why-you-should-not-use-it-for-authentication-5f47597b2611




-----


FOR THE CREATING PENDING ACTIONS AND GUARANTEEING THEY ARE THE THING CREATED BY THE USER ()
* Concerned that searching the codebase for SAVE_NOTE isn't enough. Could be created char by char, and packed with Gmail data.
* Can we checksum the event somehow, or the content, or the DOM element -> a way to verify the event created it.
* Do we let pending actions actually happen for save note? Maybe we just want to go direct to save note API request for us, at least until we pass a Security Assessment.
** Or create pending actions directly within the self contained API, so they can only be processed if created in the right place, and the only place they're created is in the user event <- THIS IS PROBABLY IT.
* Potentially once we've contained the gmail messages database in a closure, can statistically check the note isn't a known message. But the performance cost is high, and messages could be cleared.

