/*
 * ***** BEGIN LICENSE BLOCK *****
 * ActiveInbox source code. Copyright (2009) of The Inbox Foundry Ltd.
 * No reuse without express permission.
 * If you're interested in utilising or working on this code, please contact Andy Mitchell (andy@inboxfoundry.com).
 * ***** END LICENSE BLOCK ***** */



/**
 * Provides internal tracing and debugging functionality. It supports object
 * dumping, fast message logging, error/stack trace logging, and it can be easilly
 * disabled for release builds.
 *
 * TODO:
 *  Fix up all TODOLOGGER
 *  How do we handle giPrefManager, pref-manager, giLoggerFeatures, giI18N, giPrefData, giPrefManager, giRss, etc.
 *
 * @class ?
 */
var giLogger = GTD_CLASS_REGISTRY["giLogger"] = giBase.extend({
    
    logLevel: 'warn', // One of 'log', 'concern', 'warn'
    
    
    constructor: function(id) {
        this.id = id;
        this._log = [];
        
        
        this._console = new giLoggerConsole();
        
        var me = this;
        this._loadFailureTimeout = setTimeout(function() {
            me._loadFailure();
        }, 1000*60*4);
    },
    
    unload: function() {
        if( this._loadFailureTimeout ) {
            clearTimeout(this._loadFailureTimeout);
            this._loadFailureTimeout = null;
        }
        if( this._features ) {
            this._features.unload();
            this._features = null;
        }
    },
    
    _shouldLogToConsole: function (level) {
        var levels = ['log','concern','warn'];
        return !this._disabled && levels.indexOf(level) >= levels.indexOf(this.logLevel);
    },
    
    setAccount: function(account, serverAccount) {
        if( this._account!=account ) {
            this._account = account;
            this._serverAccount = serverAccount;
            this._features = new giLoggerFeatures(account, this, this._serverAccount);
        }
    },

    setGmail: function(gmail) {
        this._gmail = gmail;
    },
    
    setDocument: function(doc) {
        this._doc = doc;
    },
    
    isDisabled: function(v) {
        if( typeof v=="boolean" ) this._disabled = v;
        return this._disabled;
    },
    
    
    getFeatures: function() {
        if( !this._features ) this.throwError("account has not been set, features is null", "giLogger.getFeatures");
        return this._features;
    },
    
    /**
     * .loadSuccess was not called, which means it failed before the user can interact with the logger by any other means.
     * So invoke the popup that lets them know it failed, and optionally email us or submit to the server.
     */
    _loadFailure: function() {
        if( this._disabled ) return; // Ignore disabled accounts
        if( this._hadLoadSuccess ) return;
        
        var failPopup = new giLoggerFailPopup(this._doc, this);
    },
    
    loadSuccess: function() {      
        this._hadLoadSuccess = true;
        if( this._loadFailureTimeout ) {
            clearTimeout(this._loadFailureTimeout);
            this._loadFailureTimeout = null;
        }
    },
    
    
    
    log: function(msg, context) {
        var logMsg = "";
        if( context != undefined ) logMsg += context+': ';
        if( msg ) logMsg += msg.toString? msg.toString() : msg;

        var d = new Date();
        logMsg += " [t: "+d.getHours()+":"+d.getMinutes()+":"+d.getSeconds()+"."+d.getMilliseconds()+"]";
        
        if( this._shouldLogToConsole("log") ) this._console.log(logMsg);
    },
    
    logConcern: function(msg, context) {
        if( msg ) msg = msg.toString? msg.toString() : msg;
        msg = "[CONCERN] "+msg;
        if( this._shouldLogToConsole("concern") ) this.log(msg, context);
    },
    
    warn: function(msg, context, details) {
        var item = new giLoggerItemError(msg, context, details);
        
        this._log.push(item);

        if( this._gmail && this._gmail.rsg ) {
            this._gmail.rsg.dataContainer.dispatch(this._gmail.rsg.actionsCreator.majorWarning(message, 'giLogger.warn', {context: context, details: details}));
        }
        
        if( this._shouldLogToConsole("warn") ) this._console.log(item.toString());
    },
    
    throwError: function(msg, context, details) {
        this.warn(msg, context, details);
        
        throw new Error(msg.toString()); // After error message logged, throw an exception
    },
    

    
    getItems: function() {
        return this._log.slice();
    },
    
    toString: function() {
        var logItems = this.getItems(); // Will clone
        for( var i = 0; i < logItems.length; i++ ) {
            if( typeof logItems[i]!="string" ) {
                logItems[i] = giJson.encode(logItems[i], true);
            }
        }
        return logItems.join("\n");
    },
    
    /**
     * Wipe the logs out.
     * E.g. this may occur after a log is sent to the server.
     * @function {} ?
     */
    reset: function() {
        this._log = []; // Create anew, just incase someone was using the old one
    }
    
    
    
}, {
    
    

    
    
    COMPLEX_LOG_ITEM_TYPES: { // Matches logclientapp.php on server
        ERROR: 'error'
    }
    
});
giLogger.implement(giEventDispatcher); // Add event dispatching support
    


var giLoggerItemBase = GTD_CLASS_REGISTRY["giLoggerItemBase"] = giBase.extend({
    
    /**
     * Matches logclientapp.php on server
     * @type {String}
     */
    type: null,
    
    context: null,
    
    timeFormatted: null,
    
    
    
    constructor: function(context) {
        var d = new Date();
        this.timeFormatted = d.getHours()+":"+d.getMinutes()+":"+d.getSeconds()+"."+d.getMilliseconds();
        
        this.context = context;
        
    },
    
    toString: function() {
        return ("No .toString defined for "+this.type);
    }
    
});


var giLoggerItemError = GTD_CLASS_REGISTRY["giLoggerItemError"] = giLoggerItemBase.extend({
    
    details: null,
    
    errorMsg: null,
    
    stackTrace: null,
    
    constructor: function(msg, context, details) {
        this.base(context);
        this.type = giLogger.COMPLEX_LOG_ITEM_TYPES.ERROR;
        
        
        // Check integrity of message:
        if( !msg || !msg.toString ) msg = "";

        var error;
        if (msg.stack) {
            error = msg;
        } else {
            try {
                throw new Error();
            } catch(e) {
                error = e;
            }
        }
        this.stackTrace = "";
        if (error) {
            var stack = error.stack.toString().split("\n");
            this.stackTrace = "Exception thrown\n"+stack.slice(2).join("\n");
        }

        msg = msg.toString();
        if( details && details.toString ) this.details = details.toString();

        msg = msg.replace(/\n/g, '');
        if(context != undefined) {
            this.errorMsg = "Error in "+context+": "+msg;
        } else {
            this.errorMsg = "General Error: "+msg;
        }

    },
    
    toString: function() {
        return (this.errorMsg+" "+this.timeFormatted+"\n"+this.details+"\n"+this.stackTrace);
    }
    
});





