/*
 * ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * https://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is ActiveInbox.
 *
 * The Initial Developer of the Original Code is
 * Stephen Augenstein
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Andy Mitchell (andym@activeinboxhq.com)
 *
 * Originally developed for ActiveInbox for Gmail. https://www.activeinboxhq.com
 * Find out more about developing with this codebase at https://www.activeinboxhq.com
 * Anyone who utilises this code is required to include this license notice in all covered code.
 *
 * ***** END LICENSE BLOCK ***** */



/**
 * Event Dispatcher Mixin.
 * Gives the extending class event dispatching capabilities.
 *
 * @interface ?
 */
var giEventDispatcher = GTD_CLASS_REGISTRY["giEventDispatcher"] = giBase.extend({
    // Properties {
    /**
     * Event listener store
     * @property {Array} ?
     */
    __listeners: null,
    //}

    // Methods {
    /**
     * Registers an event listener with the EventDispatcher so that it may receive event notifications
     * @function ?
     * @param {String} eventName The event to listen for
     * @param {Function} listener The function to call on event dispatch
     * @param {Object} [thisObj] The optional object to use as "this" when the event is dispatched
     */
    addEventListener: function(eventName, listener, thisObj) {
        var me = this;
        if(this.__listeners == null) this.__listeners = {};
        if(!this.__listeners[eventName]) this.__listeners[eventName] = [];

        if( !eventName || !listener ) {
            var logger = null;
            if( this._main ) logger = this._main.logger;
            if( this._gmail ) logger = this._gmail.logger;

            if( !logger ) logger = this._logger;
            
            if(!eventName) logger.warn('No event!', 'giEventDispatcher.addEventListener');
            if(!listener) logger.throwError('No listener (' + eventName + ')', 'giEventDispatcher.addEventListener');
        }

        // Check and see if it hasn't been added before adding
        var found = false;
        for( var i = 0; i < this.__listeners[eventName].length; i++ ) {
            var l = this.__listeners[eventName][i];
            if(l.func === listener && l.thisObj===thisObj ) {
                found = true;
                break;
            }
        }
        if(!found) this.__listeners[eventName].push({func: listener, thisObj: thisObj});
        // return !found;
        return {
            unload: function () {
                me.removeEventListener(eventName, listener, thisObj);
            }
        };
    },

    /**
     * Register can event handler for the next firing of the event,
     * after the event fires once the handler is removed automatically
     * @param {String} eventName The event to listen for
     * @param {Function} listener The function to call on event dispatch
     * @param {Object} [thisObj] The optional object to use as "this" when the event is dispatched
     */
    addEventOnceListener: function (eventName, listener, thisObj) {
        var wrapped = function () {
            this.removeEventListener(eventName, wrapped, this);
            return listener.apply(thisObj, arguments);
        };
        this.addEventListener(eventName, wrapped , this);
    },

    /**
     * Removes the given listener from the EventDispatcher
     * @function ?
     * @param {String} eventName The event being listened for
     * @param {Function} listener The function that was registered as a listener (or null to remove all listeners for the context)
     */
    removeEventListener: function(eventName, listener, thisObj) {
        if(!this.hasEventListener(eventName)) return;

        for(var i = 0; i < this.__listeners[eventName].length; i++) {
            if((this.__listeners[eventName][i].func === listener || listener == null) && this.__listeners[eventName][i].thisObj===thisObj) {
                this.__listeners[eventName].splice(i, 1);
                break;
            }
        }
    },

    /**
     * Dispatches the given event to all listeners
     * @function ?
     * @param {Event} event The event to dispatch
     */
    dispatchEvent: function(event) {
        this._dispatchEvent(event.name, event);
        // Support catch-all handler
        this._dispatchEvent("*", event);
    },
    _dispatchEvent: function(name, event) {
        if(this.hasEventListener(name)) {
            // Add target property to event if it doesn't already exist
            if(typeof event['target'] == 'undefined') {
                event['target'] = this;
            }




            var listeners = []; // Clone because we want to protect the iterator from removeEventListener called mid-iteration
            for(var i = 0; i < this.__listeners[name].length; i++ ) listeners.push(this.__listeners[name][i]);
            for(var i = 0; i < listeners.length; i++ ) {
                var l = listeners[i];
                try {    // Catch errors and neutralise them; so one bad event does not bring down all listening.
                    for( var j = 0; j < this.__listeners[name].length; j++ ) {
                        if( l===this.__listeners[name][j] ) {
                            // Event still exists in the main array (has not been removed)
                            l.func.apply(l.thisObj, [event]);
                            break;
                        }
                    }
                } catch(e) {
                    var logger = null;
                    if( this._main ) logger = this._main.logger;
                    if( this._gmail ) logger = this._gmail.logger;
                    if( !logger ) logger = this._logger;
                    var funcToString = "";
                    try {
                        funcToString = (l && l.func)? l.func.toString() : "";
                    }catch(e){}
                    var message = 'event.name: '+event.name+" (handled as " + name + ")\nfunc.toString:\n"+funcToString;
                    if (logger) {
                        logger.warn(e,'giEventDispatcher.dispatchEvent', message);
                    } else if (console) {
                        console.warn(e.toString(), message);
                    }
                }
                
            }
        }
    },

    /**
     * Checks whether the EventDispatcher has any listeners registered for the
     * given event.
     * @function {Boolean} ?
     * @param {String} eventName The event to check for listeners for
     * @return Whether there are any event listeners registered
     */
    hasEventListener: function(eventName) {
        if(this.__listeners && this.__listeners[eventName] && this.__listeners[eventName].length) {
            return true;
        } else {
            return false;
        }
    },

    /**
     * Removes all listeners from the object. Can be used for clean up when you
     * don't want them to be registered any more.
     * @function {protected} ?
     */
    _removeAllListeners: function() {
        this.__listeners = null;
    }
    //}
});
